--[[
PartyCastingBars, rewritten and updated by Teazle, original code written by AnduinLothar:

	PartyCastingBars
	 	Adds party member casting bars to their unit frames.
	
	By: AnduinLothar
	
	
	$Id: PartyCastingBars.lua 4157 2006-10-13 20:12:04Z geowar $
	$Rev: 4157 $
	$LastChangedBy: geowar $
	$Date: 2006-10-13 13:12:04 -0700 (Fri, 13 Oct 2006) $
	
]]--
--[[
Fade in/out snippet by funkydude

-- Flashing in
local fade1 = flasher:CreateAnimation("Alpha")
fade1:SetDuration(0.5)
fade1:SetChange(1)
fade1:SetOrder(1)

-- Holding it visible for 1 second
fade1:SetEndDelay(1)

-- Flashing out
local fade2 = flasher:CreateAnimation("Alpha")
fade2:SetDuration(0.5)
fade2:SetChange(-1)
fade2:SetOrder(3)

-- Holding it for 1 second before calling OnFinished
fade2:SetEndDelay(1) 
]]
----------------------------
-- Create Main Handler Frame
----------------------------

local pcb = CreateFrame("STATUSBAR","PartyCastingBarTemplate",UIParent,CastingBarFrameTemplate)
-- Create all Casting Bars for party members
for i = 1,4,1 do
		pcb["party"..i] = CreateFrame("STATUSBAR","PartyMemberFrame"..i.."CastingBarFrame",_G["PartyMemberFrame"..i],"CastingBarFrameTemplate")
		pcb["party"..i]:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar") -- Set skin for rolling castinfo
		pcb["party"..i]:SetWidth(195)
		pcb["party"..i]:SetHeight(10)
		pcb["party"..i]:SetScale(0.75)
		pcb["party"..i]:SetPoint("LEFT","PartyMemberFrame"..i,"RIGHT",15,7) -- Anchor to Party Member Frame
		pcb["party"..i].Border:SetTexture("Interface\\Tooltips\\UI-StatusBar-Border") -- Set skin for frame
		pcb["party"..i].Border:SetWidth(202)
		pcb["party"..i].Border:SetHeight(18)
		pcb["party"..i].Border:ClearAllPoints()
		pcb["party"..i].Border:SetPoint("LEFT",pcb["party"..i],"LEFT",-5,0)
		pcb["party"..i].Flash:SetTexture("Interface\\AddOns\\PartyCastingBars\\Skin\\ArcaneBarFlash") -- Set custom flash texture because there is no standard that fits in WoW.
		pcb["party"..i].Flash:SetWidth(292)
		pcb["party"..i].Flash:SetHeight(34)
		pcb["party"..i].Flash:ClearAllPoints()
		pcb["party"..i].Flash:SetPoint("CENTER", pcb["party"..i], "CENTER", 0, 0)
		pcb["party"..i].animationGroup = pcb["party"..i]:CreateAnimationGroup() -- Create a group for animating fade out
		pcb["party"..i].animationGroup:SetToFinalAlpha(true)
		pcb["party"..i].Fade = pcb["party"..i].animationGroup:CreateAnimation("Alpha") -- Create the fade out
		pcb["party"..i].Fade:SetFromAlpha(1)
		pcb["party"..i].Fade:SetToAlpha(0)
		pcb["party"..i].Fade:SetDuration(0.5)
		pcb["party"..i].Icon = pcb["party"..i]:CreateTexture("PartyMemberFrame"..i.."CastingBarFrame.Icon","OVERLAY") -- Create a spell icon
		pcb["party"..i].Icon:SetPoint("LEFT",pcb["party"..i],"LEFT",-22,0) -- Anchor to Party Casting Bar Frame
		pcb["party"..i].Icon:SetWidth(16)
		pcb["party"..i].Icon:SetHeight(16)
		pcb["party"..i].Time = pcb["party"..i]:CreateFontString("PartyMemberFrame"..i.."CastingBarFrame.Time","OVERLAY","GameFontNormalSmall") -- Create a spelltimer
		pcb["party"..i].Time:SetPoint("LEFT","PartyMemberFrame"..i.."CastingBarFrame","RIGHT",7,0)
		pcb["party"..i].casting = 0
		pcb["party"..i].channeling = 0
		pcb["party"..i].startTime = 0
		pcb["party"..i].endTime = 0
		pcb["party"..i].failed = false
		pcb["party"..i].unit = "party"..i -- Callback for OnUpdate event handler
		pcb["party"..i]:RegisterEvent("UNIT_SPELLCAST_START") -- Register frame for events we care about
		pcb["party"..i]:RegisterEvent("UNIT_SPELLCAST_STOP")
		pcb["party"..i]:RegisterEvent("UNIT_SPELLCAST_FAILED")
		pcb["party"..i]:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
		pcb["party"..i]:RegisterEvent("UNIT_SPELLCAST_DELAYED")
		pcb["party"..i]:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
		pcb["party"..i]:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
		pcb["party"..i]:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
		pcb["party"..i]:Hide() -- Hide frame because we don't want to see it if noone is casting.
end
----------------------------
-- Party Member Event Handler
----------------------------
local function PartyCastingBars_OnEvent(self,event,unit,spellName)
	if unit ~= "party1" and unit ~= "party2" and unit ~= "party3" and unit ~= "party4" then -- Should really make that prettier
		return
	end
	if event == "UNIT_SPELLCAST_START" then
		local spellName, text, texture, startTime, endTime, isTradeSkill, castID, notInterruptable, spellID = UnitCastingInfo(unit);
		if (not spellName) then
			pcb[unit]:Hide()
			return
		end
		pcb[unit].failed = false
		pcb[unit].startTime = startTime / 1000
		pcb[unit].endTime = endTime / 1000
		pcb[unit].Text:SetText(spellName)
		pcb[unit]:SetMinMaxValues(pcb[unit].startTime,pcb[unit].endTime)
		pcb[unit]:SetValue(pcb[unit].startTime)
		pcb[unit].casting = 1
		pcb[unit].channeling = 0
		pcb[unit].animationGroup:Stop()
		pcb[unit]:SetAlpha(1)
		pcb[unit].Icon:SetTexture(texture)
		pcb[unit].Flash:Hide()
		pcb[unit]:Show()
		if UnitCanAttack("player", unit.."target") then
			pcb[unit]:SetStatusBarColor(1.0,0.7,0.0)
		else
			pcb[unit]:SetStatusBarColor(0.0,1.0,0.0)
		end
		return
	elseif event == "UNIT_SPELLCAST_STOP" or event == "UNIT_SPELLCAST_CHANNEL_STOP" then
		pcb[unit].Flash:Show()
		pcb[unit].animationGroup:Play()
	elseif event == "UNIT_SPELLCAST_FAILED" then
		pcb[unit]:SetStatusBarColor(1.0,0.0,0.0)
		pcb[unit].failed = true
		pcb[unit]:SetValue(pcb[unit].endTime)
		pcb[unit].Text:SetText(FAILED)
		pcb[unit].Flash:Show()
		pcb[unit].animationGroup:Play()
	elseif event == "UNIT_SPELLCAST_INTERRUPTED" then
		pcb[unit]:SetStatusBarColor(1.0,0.0,0.0)
		pcb[unit].failed = true
		pcb[unit]:SetValue(pcb[unit].endTime)
		pcb[unit].Text:SetText(INTERRUPTED)
		pcb[unit].Flash:Show()
		pcb[unit].animationGroup:Play()
	elseif event == "UNIT_SPELLCAST_DELAYED" then
		local spellName, text, texture, startTime, endTime, isTradeSkill, castID, notInterruptable, spellID = UnitCastingInfo(unit);
		if (not spellName) then
			pcb[unit]:Hide()
			return
		end
		pcb[unit].startTime = startTime / 1000
		pcb[unit].endTime = endTime / 1000
		pcb[unit].Text:SetText(spellName)
		pcb[unit]:SetMinMaxValues(pcb[unit].startTime,pcb[unit].endTime)
		pcb[unit]:SetValue(pcb[unit].startTime)
		pcb[unit].casting = 1
		pcb[unit].channeling = 0
		pcb[unit].animationGroup:Stop()
		pcb[unit]:SetAlpha(1)
		pcb[unit].Icon:SetTexture(texture)
		pcb[unit].Flash:Hide()
		pcb[unit]:Show()
		if UnitCanAttack("player", unit.."target") then
			pcb[unit]:SetStatusBarColor(1.0,0.7,0.0)
		else
			pcb[unit]:SetStatusBarColor(0.0,1.0,0.0)
		end
	elseif event == "UNIT_SPELLCAST_CHANNEL_START" then
			local spellName, text, texture, startTime, endTime, isTradeSkill, castID, notInterruptable, spellID = UnitCastingInfo(unit);
		if (not spellName) then
			pcb[unit]:Hide()
			return
		end
		pcb[unit].failed = false
		pcb[unit].startTime = startTime / 1000
		pcb[unit].endTime = endTime / 1000
		pcb[unit].Text:SetText(spellName)
		pcb[unit]:SetMinMaxValues(pcb[unit].startTime,pcb[unit].endTime)
		pcb[unit]:SetValue(pcb[unit].startTime)
		pcb[unit].channeling = 1
		pcb[unit].animationGroup:Stop()
		pcb[unit]:SetAlpha(1)
		pcb[unit].Flash:Hide()
		pcb[unit]:Show()
		if UnitCanAttack("player", unit.."target") then
			pcb[unit]:SetStatusBarColor(1.0,0.7,0.0)
		else
			pcb[unit]:SetStatusBarColor(0.0,1.0,0.0)
		end
	elseif event == "UNIT_SPELLCAST_CHANNEL_UPDATE" then
	
	end
end
local function PartyCastingBars_OnUpdate(self,elapsed,...)
--print("PARTYMEMBER_ONUPDATE",self,event,elapsed)
	local x = self:GetName()
--	print(x,_G[self:GetName()].unit)
	local unit = _G[x].unit
	if pcb[unit].failed then
		return
	end
--	if GetTime() > pcb[unit].endTime then
--		pcb[unit]:Hide()
--		return
--	end
	if pcb[unit].casting then
		local status = GetTime()
		if status > pcb[unit].endTime then
			status = pcb[unit].endTime
		end
		if status == pcb[unit].endTime then
			pcb[unit]:SetValue(status)
			if pcb[unit].failed then
				pcb[unit]:SetStatusBarColor(1.0,0.0,0.0)
			else
				pcb[unit]:SetStatusBarColor(0.0,1.0,0.0)
			end
				pcb[unit].casting = 0
			return;
		end
		pcb[unit]:SetValue(status);
		local sparkPosition = ((status - pcb[unit].startTime) / (pcb[unit].endTime - pcb[unit].startTime)) * pcb[unit]:GetWidth();
		if ( sparkPosition < 0 ) then
			sparkPosition = 0;
		end
		pcb[unit].Spark:SetPoint("CENTER", pcb[unit], "LEFT", sparkPosition, 0)
		pcb[unit].Time:SetFormattedText("%.2f %s",select(2,pcb[unit]:GetMinMaxValues()) - pcb[unit]:GetValue(),"s")
	elseif pcb[unit].channeling then
		local time = GetTime()
		if time > pcb[unit].endTime then
			time = pcb[unit].endTime
		end
		if time == bar.endTime then
			pcb[unit]:SetStatusBarColor(0.0,1.0,0.0)
			pcb[unit].channeling = 0
			return
		end
		local barValue = pcb[unit].startTime + (pcb[unit].endTime - time);
		pcb[unit]:SetValue(barValue)
		local sparkPosition = ((barValue - pcb[unit].startTime) / (pcb[unit].endTime - pcb[unit].startTime)) * pcb[unit]:GetWidth();
		pcb[unit].Spark:SetPoint("CENTER", pcb[unit], "LEFT", sparkPosition, 0)
		pcb[unit].Time:SetFormattedText("%.2f %s",pcb[unit]:GetValue() - select(1,pcb[unit]:GetMinMaxValues()),"s")
	end
end
----------------------------
-- Main Frame Event Handler
----------------------------
--[[
local function PartyCastingBarsFrame_OnEvent(self,event,...)
--	if (event == "VARIABLES_LOADED") then
		--PartyCastingBars.RegisterConfig()
--	elseif (event == "PLAYER_ENTERING_WORLD") then
	if (event == "PLAYER_ENTERING_WORLD") then
		--PartyCastingBars.CachePartyMembers()
	elseif (event == "PARTY_MEMBERS_CHANGED") then
		--PartyCastingBars.CachePartyMembers()
	elseif (event == "GROUP_ROSTER_UPDATE") then
		--PartyCastingBars.CachePartyMembers()
	elseif (event == "PLAYER_TARGET_CHANGED") then
		if (UnitExists("target")) then
--			PartyCastingBars.HostilityCache[UnitName("target")] = UnitCanAttack("player", "target")
		end
	end
end
pcbf:SetScript("OnEvent",PartyCastingBarsFrame_OnEvent)
--]]
for i = 1,4,1 do
		pcb["party"..i]:SetScript("OnEvent",PartyCastingBars_OnEvent)
		pcb["party"..i]:SetScript("OnUpdate",PartyCastingBars_OnUpdate)
end